//  Source File Name: Shred.java  1.2
/******************************************************************************/
/** Source File Name = Generate.java                                         **/
/** Licensed Materials - Property of IBM                                     **/
/** US Government Users Restricted Rights - Use, duplication or              **/
/** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.        **/
/**                                                                          **/
/**                                                                          **/
/** PURPOSE:          Decompose a XML document to database                   **/
/**                                                                          **/
/** STEPS:            Compile this file by doing: javac GenerateC.java       **/
/**                   run JDBCgenxmlC.cmd or see usage below                 **/
/**                                                                          **/
/** FUNCTIONS USED:   dxxGenXMLClob stored procedure                         **/
/**                                                                          **/
/** CONFIGURATION:    Database already created                               **/
/**                   Database must enabled with XML Extender                **/
/**                   run: dxxadm enable_db <dbname>                         **/
/**                                                                          **/
/** USAGE:            java generateC <dbname> <dadfilename> <xmlfilename>    **/
/**                                                                          **/
/******************************************************************************/


import java.sql.*;              // JDBC classes
import java.io.*;

/* DOCUMENTED */
/**
 * Licensed Materials - Property of IBM. (C) COPYRIGHT International Business Machines Corp. 1999. All Rights Reserved. US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * <h2>Purpose</h2>
 * This class calls the dxxGenXML Stored Procedure.
 *
 * @version 1.0
 */

class GenerateC {
   static
   {   
    	try {   
    		Class.forName ("COM.ibm.db2.jdbc.app.DB2Driver").newInstance ();
        }
        catch (Exception e) {   
        	System.err.println ("\n  Error loading DB2 Driver...\n" + e);
            System.exit(1);
        }
    }
    
    public static void main (String argv[])
    {   
    	try {   
    		
    		if (argv.length < 1 || argv.length > 6) {
    		  System.out.println("Usage: java Generate dbname dadfilename [-o OVERRIDE_TYPE override-string]");
    		  System.exit(1);
    		}
    		
  		String DBName = argv[0];
    		
                // read DAD
                // buffer is the temp 4k buffer to read in a while loop
		int bufSize = 4096;
		char[] buffer = new char[bufSize];
		StringBuffer StringDAD = new StringBuffer("");
            	System.out.println ("\ndad file : " + argv[1] );
            	//System.out.println ("\n result_tab : " + argv[2] );

		
		FileReader in = new FileReader(argv[1]);
		int nRead = -1;
		
                // read in a loop
		while ((nRead = in.read(buffer)) >=0) {
		  StringDAD.append(buffer, 0, nRead);
		}
		
            // get result_tab
            //String   result_tab = argv[2];

	    System.out.println ("DAD size = "+ StringDAD.length());
	    
	    System.out.println();
	    
       	    System.out.println ("Java GenXML Stored Procedure Sample");
			
            // Connect to Sample database
            
            System.out.println ("Connect to "+DBName); 

            Connection con = null;
            String url = "jdbc:db2:"+DBName;

            con = DriverManager.getConnection(url);

            // Set AutoCommit
            con.setAutoCommit(true);

            String storedProcName = "db2xml.dxxGenXMLClob";

            // prepare the CALL statement
            CallableStatement callableStmt;
            String sql = "Call " + storedProcName + "(? ? ? ? ? ? ? ?) ";
            callableStmt = con.prepareCall (sql);

            // register the output parameters
            int isValid = 0;
            int actualNumOfDocs = 0;
            int errCode = 0;
            Clob outClob ;
            String msgText = "";
            
            callableStmt.registerOutParameter (4, Types.CLOB);
            //callableStmt.setClob (4, outClob);
            callableStmt.registerOutParameter (5, Types.INTEGER);
            //callableStmt.setInt(5, isValid);
            callableStmt.registerOutParameter (6, Types.INTEGER);
            //callableStmt.setInt(6, actualNumOfDocs);
            callableStmt.registerOutParameter (7, Types.INTEGER);
            //callableStmt.setInt(7, errCode);
            callableStmt.registerOutParameter (8, Types.VARCHAR);
            //callableStmt.setString(8, msgText);

            // set all parameters
  	    callableStmt.setString(1, StringDAD.toString());
  	    //callableStmt.setString(2, result_tab);

            int  overrideType = 0;       // overrideType: NO_OVERRIDE
            String override = "";        // override value
            if (argv.length==6) {
              overrideType = ((argv[2]=="SQL_OVERRIDE"))? 1: 2;
              override = argv[3] + '\0';
            }
            callableStmt.setInt(2, overrideType);
            callableStmt.setString(3, override);

            //int maxNumOfDocs = 100;       // just set to 100 for this example 
            //callableStmt.setInt(5, maxNumOfDocs);
  	    
            // call the stored procedure
            con.setAutoCommit(false);  // Enable transactions
            try	{   
            	System.out.println ("\nCalling stored procedure : " + storedProcName );
                callableStmt.execute ();

                // Commit the transaction
                con.commit();
            } 
            catch (Exception e) {   // Rollback the transaction
                con.rollback();
                throw e; 
            }
            
            finally {   // Restore initial AutoCommit
                con.setAutoCommit(true);
            }
 
            // retrieve output parameters
            outClob = callableStmt.getClob (4);
            String outString = clobToString(outClob);
            //Reader reader = outClob.getCharacterStream();
            //Reader.read();
     
            isValid = callableStmt.getInt (5);
            actualNumOfDocs = callableStmt.getInt (6);
            errCode = callableStmt.getInt (7);
            msgText = callableStmt.getString (8);

            // display the information returned from the stored procedure
            System.out.println ("\n Document Generated:" + outString);
            System.out.println ("\n Document is valid = " + isValid);
            System.out.println ("\n Number of docs generated = " + actualNumOfDocs);
            System.out.println ("\n errCode = " + errCode);
            System.out.println (" msgText = " + msgText);
      
            callableStmt.close ();
        }
        catch (FileNotFoundException e) {
        	System.err.println (e);
        	return;
        }
        catch (IOException e) {
        	System.out.println ("Error reading DAD file" + e);
        	return;
        }
        catch (Exception e)	{   
        	System.err.println (e);
        	return;
        }
    }
    
    public static String clobToString(Clob clob) throws Exception {

	if (clob == null)
		return null;
	
    Reader reader = clob.getCharacterStream();
    int bufSize = 1024;
    char[] cbuf = new char[bufSize];
    StringBuffer strBuf = new StringBuffer();
    int numRead = reader.read(cbuf, 0, bufSize);
    while (numRead > 0) {
        strBuf.append(cbuf, 0, numRead);
        numRead = reader.read(cbuf, 0, bufSize);
    }
    
    return strBuf.toString();
}
}
